functor OclParserLexFun(structure Tokens: OclParser_TOKENS)=
   struct
    structure UserDeclarations =
      struct
(*****************************************************************************
 * su4sml --- a SML repository for managing (Secure)UML/OCL models
 *             http://projects.brucker.ch/su4sml/
 *                                                                            
 * ocl.lex --- 
 * This file is part of su4sml.
 *
 * Copyright (c) 2005-2007, ETH Zurich, Switzerland
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *
 *     * Neither the name of the copyright holders nor the names of its
 *       contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(* $Id$ *)


open Rep_OclTerm
open Rep_OclType
open Context

structure Tokens = Tokens

type pos = int * int * int
type svalue = Tokens.svalue

type ('a,'b) token = ('a,'b) Tokens.token
type lexresult= (svalue,pos) token


val pos = ref (0,0,0)

  fun eof () = Tokens.EOF((!pos,!pos))
  fun error (e,p : (int * int * int),_) = TextIO.output (TextIO.stdOut, 
							 String.concat[
								       "line ", (Int.toString (#1 p)), "/",
								       (Int.toString (#2 p - #3 p)),": ", e, "\n"
								       ])
  
 fun inputPos yypos = ((#1 (!pos), yypos - (#3(!pos)), (#3 (!pos))),
		     (#1 (!pos), yypos - (#3(!pos)), (#3 (!pos)))) 
 fun inputPos_half yypos = (#1 (!pos), yypos - (#3(!pos)), (#3 (!pos)))



end (* end of user routines *)
exception LexError (* raised if illegal leaf action tried *)
structure Internal =
	struct

datatype yyfinstate = N of int
type statedata = {fin : yyfinstate list, trans: int Vector.vector}
(* transition & final state table *)
val tab = let
fun decode s k =
  let val k' = k + k
      val hi = Char.ord(String.sub(s, k'))
      val lo = Char.ord(String.sub(s, k' + 1))
  in hi * 256 + lo end
val s = [ 
 (0,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (1,129, 
"\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\
\\000\003\001\048\001\050\000\003\000\003\000\003\000\003\000\003\
\\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\
\\000\003\000\003\000\003\000\003\000\003\000\003\000\003\000\003\
\\001\048\000\003\000\003\001\047\000\003\000\003\000\003\001\043\
\\001\042\001\041\001\040\001\039\001\038\001\034\001\032\001\022\
\\001\017\001\017\001\017\001\017\001\017\001\017\001\017\001\017\
\\001\017\001\017\001\015\001\014\001\011\001\010\001\008\001\007\
\\001\003\000\007\001\000\000\246\000\007\000\007\000\007\000\007\
\\000\007\000\007\000\007\000\007\000\007\000\007\000\007\000\236\
\\000\007\000\007\000\007\000\227\000\218\000\007\000\007\000\007\
\\000\007\000\007\000\007\000\217\000\003\000\216\000\214\000\007\
\\000\003\000\191\000\181\000\157\000\150\000\130\000\120\000\115\
\\000\007\000\090\000\007\000\007\000\087\000\082\000\079\000\044\
\\000\032\000\007\000\026\000\020\000\013\000\007\000\007\000\007\
\\000\010\000\007\000\007\000\006\000\005\000\004\000\003\000\003\
\\000\003"
),
 (7,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (9,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\009\000\000\000\000\000\000\000\000\000\009\
\\000\000\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\009\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (10,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\011\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (11,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\012\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (13,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\017\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\014\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (14,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\015\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (15,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\016\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (17,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\018\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (18,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\019\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (20,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\021\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (21,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\022\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (22,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\023\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (23,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\024\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (24,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\025\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (26,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\027\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (27,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\028\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (28,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\029\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (29,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\030\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (30,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\031\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (32,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\038\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\035\
\\000\008\000\008\000\033\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (33,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\034\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (35,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\036\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (36,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\037\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (38,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\039\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (39,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\040\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (40,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\041\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (41,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\042\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (42,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\043\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (44,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\057\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\055\000\008\
\\000\046\000\008\000\045\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (46,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\047\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (47,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\048\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (48,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\049\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (49,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\050\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (50,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\051\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (51,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\052\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (52,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\053\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (53,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\054\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (55,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\056\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (57,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\058\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (58,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\073\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\059\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (59,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\060\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (60,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\067\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\061\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (61,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\062\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (62,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\063\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (63,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\064\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (64,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\065\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (65,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\066\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (67,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\068\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (68,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\069\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (69,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\070\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (70,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\071\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (71,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\072\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (73,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\074\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (74,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\075\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (75,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\076\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (76,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\077\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (77,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\078\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (79,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\080\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (80,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\081\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (82,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\083\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (83,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\084\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (84,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\085\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (85,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\086\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (87,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\088\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (88,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\089\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (90,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\114\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\108\000\104\000\008\
\\000\008\000\008\000\008\000\097\000\091\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (91,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\092\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (92,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\093\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (93,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\094\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (94,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\095\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (95,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\096\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (97,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\098\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (98,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\099\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (99,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\100\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (100,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\101\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (101,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\102\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (102,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\103\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (104,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\106\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\105\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (106,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\107\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (108,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\109\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (109,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\110\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (110,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\111\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (111,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\112\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (112,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\113\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (115,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\116\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (116,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\117\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (117,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\118\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (118,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\119\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (120,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\126\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\121\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (121,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\122\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (122,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\123\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (123,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\124\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (124,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\125\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (126,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\127\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (127,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\128\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (128,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\129\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (130,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\147\000\008\000\136\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\131\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (131,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\132\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (132,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\133\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (133,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\134\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (134,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\135\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (136,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\137\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (137,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\145\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\138\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (138,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\139\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (139,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\140\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (140,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\141\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (141,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\142\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (142,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\143\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (143,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\144\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (145,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\146\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (147,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\148\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (148,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\149\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (150,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\151\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (151,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\156\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\152\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (152,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\153\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (153,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\154\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (154,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\155\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (157,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\177\000\008\000\008\000\158\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (158,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\172\000\008\000\159\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (159,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\164\000\160\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (160,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\161\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (161,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\162\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (162,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\163\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (164,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\165\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (165,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\166\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (166,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\167\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (167,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\168\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (168,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\169\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (169,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\170\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (170,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\171\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (172,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\173\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (173,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\174\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (174,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\175\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (175,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\176\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (177,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\178\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (178,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\179\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (179,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\180\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (181,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\185\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\182\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (182,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\183\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (183,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\184\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (185,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\186\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (186,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\187\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (187,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\188\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (188,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\189\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (189,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\190\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (191,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\211\000\008\
\\000\008\000\008\000\008\000\201\000\192\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (192,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\193\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (193,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\194\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (194,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\195\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (195,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\196\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (196,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\197\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (197,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\198\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (198,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\199\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (199,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\200\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (201,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\202\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (202,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\203\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (203,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\204\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (204,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\205\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (205,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\206\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (206,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\207\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (207,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\208\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (208,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\209\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (209,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\210\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (211,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\213\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\212\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (214,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\215\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (218,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\219\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (219,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\220\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (220,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\221\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (221,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\222\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (222,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\223\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (223,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\224\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (224,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\225\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (225,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\226\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (227,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\228\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (228,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\230\000\008\000\008\000\229\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (230,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\231\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (231,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\232\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (232,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\233\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (233,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\234\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (234,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\235\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (236,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\237\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (237,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\238\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (238,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\239\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (239,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\240\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (240,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\241\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (241,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\242\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (242,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\243\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (243,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\244\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (244,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\245\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (246,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\247\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (247,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\248\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (248,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\249\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (249,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\250\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (250,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\251\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (251,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\252\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (252,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\253\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (253,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\254\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (254,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\255\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (256,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\001\001\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (257,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\009\000\009\000\009\000\009\000\009\000\009\000\009\000\009\
\\000\009\000\009\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\008\
\\000\000\000\008\000\008\000\008\000\008\000\008\000\008\001\002\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\008\000\008\000\008\000\008\000\008\
\\000\008\000\008\000\008\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (259,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\001\004\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (260,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\001\005\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (261,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\001\006\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (264,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\001\009\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (267,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\001\013\001\012\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (271,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\001\016\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (273,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\001\019\000\000\
\\001\018\001\018\001\018\001\018\001\018\001\018\001\018\001\018\
\\001\018\001\018\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (275,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\001\020\001\020\001\020\001\020\001\020\001\020\001\020\001\020\
\\001\020\001\020\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (276,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\001\020\001\020\001\020\001\020\001\020\001\020\001\020\001\020\
\\001\020\001\020\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\001\021\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\001\021\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (277,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\001\021\001\021\001\021\001\021\001\021\001\021\001\021\001\021\
\\001\021\001\021\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (278,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\001\023\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (279,129, 
"\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\025\001\024\001\024\001\024\001\024\001\031\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024"
),
 (280,129, 
"\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\025\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024"
),
 (281,129, 
"\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\029\001\026\001\026\001\026\001\026\001\028\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026\001\026\001\026\001\026\001\026\001\026\001\026\001\026\
\\001\026"
),
 (282,129, 
"\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\025\001\024\001\024\001\024\001\024\001\027\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024"
),
 (283,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\001\026\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (285,129, 
"\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\025\001\024\001\024\001\024\001\024\001\030\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024\001\024\001\024\001\024\001\024\001\024\001\024\001\024\
\\001\024"
),
 (288,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\001\033\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (290,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\001\036\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\001\035\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (292,129, 
"\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\037\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036\001\036\001\036\001\036\001\036\001\036\001\036\001\036\
\\001\036"
),
 (299,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\001\044\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\001\044\000\000\000\000\000\000\000\000\000\000\000\000\001\046\
\\000\000\000\000\000\000\000\000\000\000\001\045\001\045\001\045\
\\001\044\001\044\001\044\001\044\001\044\001\044\001\044\001\044\
\\001\044\001\044\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\001\044\001\044\001\044\001\044\001\044\001\044\001\044\
\\001\044\001\044\001\044\001\044\001\044\001\044\001\044\001\044\
\\001\044\001\044\001\044\001\044\001\044\001\044\001\044\001\044\
\\001\044\001\044\001\044\000\000\000\000\000\000\000\000\001\044\
\\000\000\001\044\001\044\001\044\001\044\001\044\001\044\001\044\
\\001\044\001\044\001\044\001\044\001\044\001\044\001\044\001\044\
\\001\044\001\044\001\044\001\044\001\044\001\044\001\044\001\044\
\\001\044\001\044\001\044\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (301,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\001\045\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\001\045\000\000\000\000\000\000\000\000\000\000\000\000\001\046\
\\000\000\000\000\000\000\000\000\000\000\001\045\001\045\001\045\
\\001\045\001\045\001\045\001\045\001\045\001\045\001\045\001\045\
\\001\045\001\045\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\001\045\001\045\001\045\001\045\001\045\001\045\001\045\
\\001\045\001\045\001\045\001\045\001\045\001\045\001\045\001\045\
\\001\045\001\045\001\045\001\045\001\045\001\045\001\045\001\045\
\\001\045\001\045\001\045\000\000\000\000\000\000\000\000\001\045\
\\000\000\001\045\001\045\001\045\001\045\001\045\001\045\001\045\
\\001\045\001\045\001\045\001\045\001\045\001\045\001\045\001\045\
\\001\045\001\045\001\045\001\045\001\045\001\045\001\045\001\045\
\\001\045\001\045\001\045\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
 (304,129, 
"\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\001\049\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\001\049\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
\\000\000"
),
(0, 0, "")]
fun f(n, i, x) = (n, Vector.tabulate(i, decode x)) 
val s = map f (rev (tl (rev s))) 
exception LexHackingError 
fun look ((j,x)::r, i: int) = if i = j then x else look(r, i) 
  | look ([], i) = raise LexHackingError
fun g {fin=x, trans=i} = {fin=x, trans=look(s,i)} 
in Vector.fromList(map g 
[{fin = [], trans = 0},
{fin = [], trans = 1},
{fin = [], trans = 1},
{fin = [(N 469)], trans = 0},
{fin = [(N 219),(N 469)], trans = 0},
{fin = [(N 221),(N 469)], trans = 0},
{fin = [(N 217),(N 469)], trans = 0},
{fin = [(N 467),(N 469)], trans = 7},
{fin = [(N 467)], trans = 7},
{fin = [(N 467)], trans = 9},
{fin = [(N 467),(N 469)], trans = 10},
{fin = [(N 467)], trans = 11},
{fin = [(N 354),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 13},
{fin = [(N 467)], trans = 14},
{fin = [(N 467)], trans = 15},
{fin = [(N 211),(N 467)], trans = 7},
{fin = [(N 467)], trans = 17},
{fin = [(N 467)], trans = 18},
{fin = [(N 206),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 20},
{fin = [(N 467)], trans = 21},
{fin = [(N 467)], trans = 22},
{fin = [(N 467)], trans = 23},
{fin = [(N 467)], trans = 24},
{fin = [(N 242),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 26},
{fin = [(N 467)], trans = 27},
{fin = [(N 467)], trans = 28},
{fin = [(N 467)], trans = 29},
{fin = [(N 467)], trans = 30},
{fin = [(N 249),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 32},
{fin = [(N 467)], trans = 33},
{fin = [(N 196),(N 467)], trans = 7},
{fin = [(N 467)], trans = 35},
{fin = [(N 467)], trans = 36},
{fin = [(N 201),(N 467)], trans = 7},
{fin = [(N 467)], trans = 38},
{fin = [(N 467)], trans = 39},
{fin = [(N 467)], trans = 40},
{fin = [(N 467)], trans = 41},
{fin = [(N 467)], trans = 42},
{fin = [(N 192),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 44},
{fin = [(N 350),(N 467)], trans = 7},
{fin = [(N 467)], trans = 46},
{fin = [(N 467)], trans = 47},
{fin = [(N 467)], trans = 48},
{fin = [(N 467)], trans = 49},
{fin = [(N 467)], trans = 50},
{fin = [(N 467)], trans = 51},
{fin = [(N 467)], trans = 52},
{fin = [(N 467)], trans = 53},
{fin = [(N 400),(N 467)], trans = 7},
{fin = [(N 467)], trans = 55},
{fin = [(N 279),(N 467)], trans = 7},
{fin = [(N 467)], trans = 57},
{fin = [(N 467)], trans = 58},
{fin = [(N 467)], trans = 59},
{fin = [(N 467)], trans = 60},
{fin = [(N 467)], trans = 61},
{fin = [(N 467)], trans = 62},
{fin = [(N 467)], trans = 63},
{fin = [(N 467)], trans = 64},
{fin = [(N 467)], trans = 65},
{fin = [(N 300),(N 467)], trans = 7},
{fin = [(N 467)], trans = 67},
{fin = [(N 467)], trans = 68},
{fin = [(N 467)], trans = 69},
{fin = [(N 467)], trans = 70},
{fin = [(N 467)], trans = 71},
{fin = [(N 312),(N 467)], trans = 7},
{fin = [(N 467)], trans = 73},
{fin = [(N 467)], trans = 74},
{fin = [(N 467)], trans = 75},
{fin = [(N 467)], trans = 76},
{fin = [(N 467)], trans = 77},
{fin = [(N 322),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 79},
{fin = [(N 467)], trans = 80},
{fin = [(N 366),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 82},
{fin = [(N 467)], trans = 83},
{fin = [(N 467)], trans = 84},
{fin = [(N 467)], trans = 85},
{fin = [(N 372),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 87},
{fin = [(N 467)], trans = 88},
{fin = [(N 184),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 90},
{fin = [(N 467)], trans = 91},
{fin = [(N 467)], trans = 92},
{fin = [(N 467)], trans = 93},
{fin = [(N 467)], trans = 94},
{fin = [(N 467)], trans = 95},
{fin = [(N 235),(N 467)], trans = 7},
{fin = [(N 467)], trans = 97},
{fin = [(N 467)], trans = 98},
{fin = [(N 467)], trans = 99},
{fin = [(N 467)], trans = 100},
{fin = [(N 467)], trans = 101},
{fin = [(N 467)], trans = 102},
{fin = [(N 288),(N 467)], trans = 7},
{fin = [(N 171),(N 467)], trans = 104},
{fin = [(N 180),(N 467)], trans = 7},
{fin = [(N 467)], trans = 106},
{fin = [(N 176),(N 467)], trans = 7},
{fin = [(N 467)], trans = 108},
{fin = [(N 467)], trans = 109},
{fin = [(N 467)], trans = 110},
{fin = [(N 467)], trans = 111},
{fin = [(N 467)], trans = 112},
{fin = [(N 362),(N 467)], trans = 7},
{fin = [(N 168),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 115},
{fin = [(N 467)], trans = 116},
{fin = [(N 467)], trans = 117},
{fin = [(N 467)], trans = 118},
{fin = [(N 227),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 120},
{fin = [(N 467)], trans = 121},
{fin = [(N 467)], trans = 122},
{fin = [(N 467)], trans = 123},
{fin = [(N 467)], trans = 124},
{fin = [(N 264),(N 467)], trans = 7},
{fin = [(N 467)], trans = 126},
{fin = [(N 467)], trans = 127},
{fin = [(N 467)], trans = 128},
{fin = [(N 165),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 130},
{fin = [(N 467)], trans = 131},
{fin = [(N 467)], trans = 132},
{fin = [(N 467)], trans = 133},
{fin = [(N 467)], trans = 134},
{fin = [(N 275),(N 467)], trans = 7},
{fin = [(N 467)], trans = 136},
{fin = [(N 436),(N 467)], trans = 137},
{fin = [(N 467)], trans = 138},
{fin = [(N 467)], trans = 139},
{fin = [(N 467)], trans = 140},
{fin = [(N 467)], trans = 141},
{fin = [(N 467)], trans = 142},
{fin = [(N 467)], trans = 143},
{fin = [(N 159),(N 467)], trans = 7},
{fin = [(N 467)], trans = 145},
{fin = [(N 148),(N 467)], trans = 7},
{fin = [(N 467)], trans = 147},
{fin = [(N 467)], trans = 148},
{fin = [(N 142),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 150},
{fin = [(N 467)], trans = 151},
{fin = [(N 467)], trans = 152},
{fin = [(N 467)], trans = 153},
{fin = [(N 467)], trans = 154},
{fin = [(N 137),(N 467)], trans = 7},
{fin = [(N 130),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 157},
{fin = [(N 467)], trans = 158},
{fin = [(N 467)], trans = 159},
{fin = [(N 467)], trans = 160},
{fin = [(N 467)], trans = 161},
{fin = [(N 467)], trans = 162},
{fin = [(N 126),(N 467)], trans = 7},
{fin = [(N 467)], trans = 164},
{fin = [(N 467)], trans = 165},
{fin = [(N 467)], trans = 166},
{fin = [(N 467)], trans = 167},
{fin = [(N 467)], trans = 168},
{fin = [(N 467)], trans = 169},
{fin = [(N 467)], trans = 170},
{fin = [(N 412),(N 467)], trans = 7},
{fin = [(N 467)], trans = 172},
{fin = [(N 467)], trans = 173},
{fin = [(N 467)], trans = 174},
{fin = [(N 467)], trans = 175},
{fin = [(N 257),(N 467)], trans = 7},
{fin = [(N 467)], trans = 177},
{fin = [(N 467)], trans = 178},
{fin = [(N 467)], trans = 179},
{fin = [(N 378),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 181},
{fin = [(N 467)], trans = 182},
{fin = [(N 467)], trans = 183},
{fin = [(N 118),(N 467)], trans = 7},
{fin = [(N 467)], trans = 185},
{fin = [(N 467)], trans = 186},
{fin = [(N 467)], trans = 187},
{fin = [(N 467)], trans = 188},
{fin = [(N 467)], trans = 189},
{fin = [(N 432),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 191},
{fin = [(N 467)], trans = 192},
{fin = [(N 467)], trans = 193},
{fin = [(N 467)], trans = 194},
{fin = [(N 467)], trans = 195},
{fin = [(N 467)], trans = 196},
{fin = [(N 467)], trans = 197},
{fin = [(N 467)], trans = 198},
{fin = [(N 467)], trans = 199},
{fin = [(N 389),(N 467)], trans = 7},
{fin = [(N 467)], trans = 201},
{fin = [(N 467)], trans = 202},
{fin = [(N 467)], trans = 203},
{fin = [(N 467)], trans = 204},
{fin = [(N 467)], trans = 205},
{fin = [(N 467)], trans = 206},
{fin = [(N 467)], trans = 207},
{fin = [(N 467)], trans = 208},
{fin = [(N 467)], trans = 209},
{fin = [(N 424),(N 467)], trans = 7},
{fin = [(N 467)], trans = 211},
{fin = [(N 268),(N 467)], trans = 7},
{fin = [(N 347),(N 467)], trans = 7},
{fin = [(N 110),(N 469)], trans = 214},
{fin = [(N 113)], trans = 0},
{fin = [(N 108),(N 469)], trans = 0},
{fin = [(N 106),(N 469)], trans = 0},
{fin = [(N 467),(N 469)], trans = 218},
{fin = [(N 467)], trans = 219},
{fin = [(N 467)], trans = 220},
{fin = [(N 467)], trans = 221},
{fin = [(N 94),(N 467)], trans = 222},
{fin = [(N 467)], trans = 223},
{fin = [(N 467)], trans = 224},
{fin = [(N 467)], trans = 225},
{fin = [(N 104),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 227},
{fin = [(N 467)], trans = 228},
{fin = [(N 88),(N 467)], trans = 7},
{fin = [(N 467)], trans = 230},
{fin = [(N 467)], trans = 231},
{fin = [(N 467)], trans = 232},
{fin = [(N 467)], trans = 233},
{fin = [(N 467)], trans = 234},
{fin = [(N 84),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 236},
{fin = [(N 467)], trans = 237},
{fin = [(N 467)], trans = 238},
{fin = [(N 467)], trans = 239},
{fin = [(N 467)], trans = 240},
{fin = [(N 467)], trans = 241},
{fin = [(N 467)], trans = 242},
{fin = [(N 467)], trans = 243},
{fin = [(N 467)], trans = 244},
{fin = [(N 75),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 246},
{fin = [(N 467)], trans = 247},
{fin = [(N 467)], trans = 248},
{fin = [(N 467)], trans = 249},
{fin = [(N 467)], trans = 250},
{fin = [(N 467)], trans = 251},
{fin = [(N 467)], trans = 252},
{fin = [(N 467)], trans = 253},
{fin = [(N 467)], trans = 254},
{fin = [(N 64),(N 467)], trans = 7},
{fin = [(N 467),(N 469)], trans = 256},
{fin = [(N 467)], trans = 257},
{fin = [(N 53),(N 467)], trans = 7},
{fin = [(N 469)], trans = 259},
{fin = [], trans = 260},
{fin = [], trans = 261},
{fin = [(N 49)], trans = 0},
{fin = [(N 42),(N 469)], trans = 0},
{fin = [(N 332),(N 469)], trans = 264},
{fin = [(N 337)], trans = 0},
{fin = [(N 40),(N 469)], trans = 0},
{fin = [(N 334),(N 469)], trans = 267},
{fin = [(N 343)], trans = 0},
{fin = [(N 340)], trans = 0},
{fin = [(N 38),(N 469)], trans = 0},
{fin = [(N 33),(N 469)], trans = 271},
{fin = [(N 36)], trans = 0},
{fin = [(N 448),(N 469)], trans = 273},
{fin = [(N 448)], trans = 273},
{fin = [], trans = 275},
{fin = [(N 445)], trans = 276},
{fin = [(N 445)], trans = 277},
{fin = [(N 328),(N 469)], trans = 278},
{fin = [], trans = 279},
{fin = [], trans = 280},
{fin = [], trans = 281},
{fin = [], trans = 282},
{fin = [], trans = 283},
{fin = [(N 21)], trans = 0},
{fin = [], trans = 285},
{fin = [(N 21)], trans = 283},
{fin = [], trans = 279},
{fin = [(N 28),(N 469)], trans = 288},
{fin = [(N 31)], trans = 0},
{fin = [(N 324),(N 469)], trans = 290},
{fin = [(N 26)], trans = 0},
{fin = [], trans = 292},
{fin = [(N 9)], trans = 0},
{fin = [(N 23),(N 469)], trans = 0},
{fin = [(N 330),(N 469)], trans = 0},
{fin = [(N 326),(N 469)], trans = 0},
{fin = [(N 215),(N 469)], trans = 0},
{fin = [(N 213),(N 469)], trans = 0},
{fin = [(N 469)], trans = 299},
{fin = [], trans = 299},
{fin = [], trans = 301},
{fin = [(N 461)], trans = 0},
{fin = [(N 44),(N 469)], trans = 0},
{fin = [(N 4),(N 469)], trans = 304},
{fin = [(N 4)], trans = 304},
{fin = [(N 1)], trans = 0}])
end
structure StartStates =
	struct
	datatype yystartstate = STARTSTATE of int

(* start state definitions *)

val INITIAL = STARTSTATE 1;

end
type result = UserDeclarations.lexresult
	exception LexerError (* raised if illegal leaf action tried *)
end

fun makeLexer yyinput =
let	val yygone0=1
	val yyb = ref "\n" 		(* buffer *)
	val yybl = ref 1		(*buffer length *)
	val yybufpos = ref 1		(* location of next character to use *)
	val yygone = ref yygone0	(* position in file of beginning of buffer *)
	val yydone = ref false		(* eof found yet? *)
	val yybegin = ref 1		(*Current 'start state' for lexer *)

	val YYBEGIN = fn (Internal.StartStates.STARTSTATE x) =>
		 yybegin := x

fun lex () : Internal.result =
let fun continue() = lex() in
  let fun scan (s,AcceptingLeaves : Internal.yyfinstate list list,l,i0) =
	let fun action (i,nil) = raise LexError
	| action (i,nil::l) = action (i-1,l)
	| action (i,(node::acts)::l) =
		case node of
		    Internal.N yyk => 
			(let fun yymktext() = substring(!yyb,i0,i-i0)
			     val yypos = i0+ !yygone
			open UserDeclarations Internal.StartStates
 in (yybufpos := i; case yyk of 

			(* Application actions *)

  1 => (pos := ((#1 (!pos)) + 1, yypos - (#3(!pos)),yypos  ); lex())
| 104 => let val yytext=yymktext() in Tokens.TUPLE_TYPE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 106 => let val yytext=yymktext() in Tokens.BRACKET_OPEN(yytext,inputPos_half yypos,inputPos_half yypos) end
| 108 => let val yytext=yymktext() in Tokens.BRACKET_CLOSE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 110 => let val yytext=yymktext() in Tokens.CARAT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 113 => let val yytext=yymktext() in Tokens.DBL_CARAT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 118 => (Tokens.BODY(body,inputPos_half yypos,inputPos_half yypos))
| 126 => let val yytext=yymktext() in Tokens.CONTEXT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 130 => let val yytext=yymktext() in Tokens.DEF(yytext,inputPos_half yypos,inputPos_half yypos) end
| 137 => (Tokens.DERIVE(derive,inputPos_half yypos,inputPos_half yypos))
| 142 => let val yytext=yymktext() in Tokens.ELSE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 148 => let val yytext=yymktext() in Tokens.ENDIF(yytext,inputPos_half yypos,inputPos_half yypos) end
| 159 => let val yytext=yymktext() in Tokens.ENDPACKAGE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 165 => let val yytext=yymktext() in Tokens.FALSE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 168 => let val yytext=yymktext() in Tokens.IF(yytext,inputPos_half yypos,inputPos_half yypos) end
| 171 => let val yytext=yymktext() in Tokens.IN(yytext,inputPos_half yypos,inputPos_half yypos) end
| 176 => (Tokens.INIT(init,inputPos_half yypos,inputPos_half yypos))
| 180 => let val yytext=yymktext() in Tokens.INV(yytext,inputPos_half yypos,inputPos_half yypos) end
| 184 => let val yytext=yymktext() in Tokens.LET(yytext,inputPos_half yypos,inputPos_half yypos) end
| 192 => let val yytext=yymktext() in Tokens.PACKAGE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 196 => (Tokens.PRE(pre,inputPos_half yypos,inputPos_half yypos))
| 201 => (Tokens.POST(post,inputPos_half yypos,inputPos_half yypos))
| 206 => let val yytext=yymktext() in Tokens.THEN(yytext,inputPos_half yypos,inputPos_half yypos) end
| 21 => (lex())
| 211 => let val yytext=yymktext() in Tokens.TRUE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 213 => let val yytext=yymktext() in Tokens.PAREN_OPEN(yytext,inputPos_half yypos,inputPos_half yypos) end
| 215 => let val yytext=yymktext() in Tokens.PAREN_CLOSE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 217 => let val yytext=yymktext() in Tokens.BRACE_OPEN(yytext,inputPos_half yypos,inputPos_half yypos) end
| 219 => let val yytext=yymktext() in Tokens.BRACE_CLOSE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 221 => let val yytext=yymktext() in Tokens.VERTICAL_BAR(yytext,inputPos_half yypos,inputPos_half yypos) end
| 227 => let val yytext=yymktext() in Tokens.GUARD(yytext,inputPos_half yypos,inputPos_half yypos) end
| 23 => let val yytext=yymktext() in Tokens.COMMA(yytext,inputPos_half yypos,inputPos_half yypos) end
| 235 => let val yytext=yymktext() in Tokens.ITERATE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 242 => let val yytext=yymktext() in Tokens.SELECT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 249 => let val yytext=yymktext() in Tokens.REJECT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 257 => let val yytext=yymktext() in Tokens.COLLECT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 26 => let val yytext=yymktext() in Tokens.ARROW_RIGHT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 264 => let val yytext=yymktext() in Tokens.FORALL(yytext,inputPos_half yypos,inputPos_half yypos) end
| 268 => let val yytext=yymktext() in Tokens.ANY(yytext,inputPos_half yypos,inputPos_half yypos) end
| 275 => let val yytext=yymktext() in Tokens.EXISTS(yytext,inputPos_half yypos,inputPos_half yypos) end
| 279 => let val yytext=yymktext() in Tokens.ONE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 28 => let val yytext=yymktext() in Tokens.DOT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 288 => let val yytext=yymktext() in Tokens.ISUNIQUE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 300 => let val yytext=yymktext() in Tokens.OCLISTYPEOF(yytext,inputPos_half yypos,inputPos_half yypos) end
| 31 => let val yytext=yymktext() in Tokens.DBL_DOT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 312 => let val yytext=yymktext() in Tokens.OCLISKINDOF(yytext,inputPos_half yypos,inputPos_half yypos) end
| 322 => let val yytext=yymktext() in Tokens.OCLASTYPE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 324 => let val yytext=yymktext() in Tokens.MINUS(yytext,inputPos_half yypos,inputPos_half yypos) end
| 326 => let val yytext=yymktext() in Tokens.STAR(yytext,inputPos_half yypos,inputPos_half yypos) end
| 328 => let val yytext=yymktext() in Tokens.SLASH(yytext,inputPos_half yypos,inputPos_half yypos) end
| 33 => let val yytext=yymktext() in Tokens.COLON(yytext,inputPos_half yypos,inputPos_half yypos) end
| 330 => let val yytext=yymktext() in Tokens.PLUS(yytext,inputPos_half yypos,inputPos_half yypos) end
| 332 => let val yytext=yymktext() in Tokens.REL_GT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 334 => let val yytext=yymktext() in Tokens.REL_LT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 337 => let val yytext=yymktext() in Tokens.REL_GTE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 340 => let val yytext=yymktext() in Tokens.REL_LTE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 343 => let val yytext=yymktext() in Tokens.REL_NOTEQUAL(yytext,inputPos_half yypos,inputPos_half yypos) end
| 347 => let val yytext=yymktext() in Tokens.LOG_AND(yytext,inputPos_half yypos,inputPos_half yypos) end
| 350 => let val yytext=yymktext() in Tokens.LOG_OR(yytext,inputPos_half yypos,inputPos_half yypos) end
| 354 => let val yytext=yymktext() in Tokens.LOG_XOR(yytext,inputPos_half yypos,inputPos_half yypos) end
| 36 => let val yytext=yymktext() in Tokens.DBL_COLON(yytext,inputPos_half yypos,inputPos_half yypos) end
| 362 => let val yytext=yymktext() in Tokens.LOG_IMPL(yytext,inputPos_half yypos,inputPos_half yypos) end
| 366 => let val yytext=yymktext() in Tokens.NOT(yytext,inputPos_half yypos,inputPos_half yypos) end
| 372 => let val yytext=yymktext() in Tokens.MODEL(yytext,inputPos_half yypos,inputPos_half yypos) end
| 378 => let val yytext=yymktext() in Tokens.CLASS(yytext,inputPos_half yypos,inputPos_half yypos) end
| 38 => let val yytext=yymktext() in Tokens.SEMI_COLON(yytext,inputPos_half yypos,inputPos_half yypos) end
| 389 => let val yytext=yymktext() in Tokens.ATTRIBUTES(yytext,inputPos_half yypos,inputPos_half yypos) end
| 4 => (pos := (#1 (!pos), yypos - (#3(!pos)), (#3 (!pos))); lex())
| 40 => let val yytext=yymktext() in Tokens.EQUALS(yytext,inputPos_half yypos,inputPos_half yypos) end
| 400 => let val yytext=yymktext() in Tokens.OPERATIONS(yytext,inputPos_half yypos,inputPos_half yypos) end
| 412 => let val yytext=yymktext() in Tokens.CONSTRAINTS(yytext,inputPos_half yypos,inputPos_half yypos) end
| 42 => let val yytext=yymktext() in Tokens.QUESTION_MARK(yytext,inputPos_half yypos,inputPos_half yypos) end
| 424 => let val yytext=yymktext() in Tokens.ASSOCIATIONS(yytext,inputPos_half yypos,inputPos_half yypos) end
| 432 => let val yytext=yymktext() in Tokens.BETWEEN(yytext,inputPos_half yypos,inputPos_half yypos) end
| 436 => let val yytext=yymktext() in Tokens.END(yytext,inputPos_half yypos,inputPos_half yypos) end
| 44 => let val yytext=yymktext() in Tokens.HASH(yytext,inputPos_half yypos,inputPos_half yypos) end
| 445 => let val yytext=yymktext() in Tokens.REAL_LITERAL(yytext,inputPos_half yypos,inputPos_half yypos) end
| 448 => let val yytext=yymktext() in Tokens.INTEGER_LITERAL(yytext,inputPos_half yypos,inputPos_half yypos) end
| 461 => let val yytext=yymktext() in Tokens.STRING_LITERAL(yytext,inputPos_half yypos,inputPos_half yypos) end
| 467 => let val yytext=yymktext() in Tokens.SIMPLE_NAME(yytext,inputPos_half yypos,inputPos_half yypos) end
| 469 => let val yytext=yymktext() in error ("ignoring bad character "^yytext,
		    ((#1 (!pos), yypos - (#3(!pos)), (#3 (!pos)))),
		    ((#1 (!pos), yypos - (#3(!pos)), (#3 (!pos)))));
             lex() end
| 49 => let val yytext=yymktext() in Tokens.AT_PRE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 53 => let val yytext=yymktext() in Tokens.BAG(yytext,inputPos_half yypos,inputPos_half yypos) end
| 64 => let val yytext=yymktext() in Tokens.COLLECTION(yytext,inputPos_half yypos,inputPos_half yypos) end
| 75 => let val yytext=yymktext() in Tokens.ORDERED_SET(yytext,inputPos_half yypos,inputPos_half yypos) end
| 84 => let val yytext=yymktext() in Tokens.SEQUENCE(yytext,inputPos_half yypos,inputPos_half yypos) end
| 88 => let val yytext=yymktext() in Tokens.SET(yytext,inputPos_half yypos,inputPos_half yypos) end
| 9 => (pos := ((#1 (!pos)) + 1, yypos - (#3(!pos)),yypos  ); lex())
| 94 => let val yytext=yymktext() in Tokens.TUPLE(yytext,inputPos_half yypos,inputPos_half yypos) end
| _ => raise Internal.LexerError

		) end )

	val {fin,trans} = Vector.sub(Internal.tab, s)
	val NewAcceptingLeaves = fin::AcceptingLeaves
	in if l = !yybl then
	     if trans = #trans(Vector.sub(Internal.tab,0))
	       then action(l,NewAcceptingLeaves
) else	    let val newchars= if !yydone then "" else yyinput 1024
	    in if (size newchars)=0
		  then (yydone := true;
		        if (l=i0) then UserDeclarations.eof ()
		                  else action(l,NewAcceptingLeaves))
		  else (if i0=l then yyb := newchars
		     else yyb := substring(!yyb,i0,l-i0)^newchars;
		     yygone := !yygone+i0;
		     yybl := size (!yyb);
		     scan (s,AcceptingLeaves,l-i0,0))
	    end
	  else let val NewChar = Char.ord(CharVector.sub(!yyb,l))
		val NewChar = if NewChar<128 then NewChar else 128
		val NewState = Vector.sub(trans, NewChar)
		in if NewState=0 then action(l,NewAcceptingLeaves)
		else scan(NewState,NewAcceptingLeaves,l+1,i0)
	end
	end
(*
	val start= if substring(!yyb,!yybufpos-1,1)="\n"
then !yybegin+1 else !yybegin
*)
	in scan(!yybegin (* start *),nil,!yybufpos,!yybufpos)
    end
end
  in lex
  end
end
